-module(state).
-export([main/0]).

% Simple server that can receive two types of messages:
%    {hello, From, Name}, where From is the PID of the sender
%    {reqs, From},  where From is the PID of the sender
% If we get {hello, From, Name}:
%    - Already communicated with From and same Name return: "Already talked with you"
%    - Already communicated with From and diff Name return: "No you're not"
%    - First communication with From return: "Hello mate!"
% If we get {reqs, From}, we return the total number of contacts we have until that point.

server(Contacts,List) ->
    receive
        {hello, From, Name} ->
	    search(Contacts,List,List,From,Name);
        {reqs, From} ->
            From ! Contacts,
	    server(Contacts,List)
    end.

search(Contacts,OldList,[],From,Name) ->
   From ! "Hello mate!",
   server(Contacts+1, [{From,Name} | OldList]);
search(Contacts,OldList,[{From,Name}|_Tail],From,Name) ->
   From ! "Already talked with you",
   server(Contacts,OldList);
search(Contacts,OldList,[{From,_OtherName}|_Tail],From,_Name) ->
   From ! "No you're not",
   server(Contacts,OldList);
search(Contacts,OldList,[_Head|Tail],From,Name) ->
   search(Contacts,OldList,Tail,From,Name).


client(Server,Name,OtherName) ->
   Server ! { hello,self(),Name},
   receive Msg1 -> io:fwrite("~s~n", [Msg1]) end,
   Server ! { hello, self(),OtherName},
   receive Msg2 -> io:fwrite("~s~n", [Msg2]) end.
    

main() ->
    Server = spawn(fun() -> server(0,[]) end),
    Xs = lists:seq(1, 10), % Creates a list from 1 to 10
    [ spawn(fun() -> client(Server,X,X+1) end) || X <- Xs],
    ok.
